@@ -1,28 +1,29 @@
-/*
+/*-
  * SSLsplit - transparent SSL/TLS interception
- * Copyright (c) 2009-2016, Daniel Roethlisberger <daniel@roe.ch>
+ * https://www.roe.ch/SSLsplit
+ *
+ * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
- * http://www.roe.ch/SSLsplit
  *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions, and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "ssl.h"
@@ -68,13 +69,16 @@ cachemgr_teardown(void)
 START_TEST(cache_ssess_01)
 {
 	SSL_SESSION *s1, *s2;
+	const unsigned char* session_id;
+	unsigned int len;
 
 	s1 = ssl_session_from_file(TMP_SESS_FILE);
 	fail_unless(!!s1, "creating session failed");
 	fail_unless(ssl_session_is_valid(s1), "session invalid");
 
 	cachemgr_ssess_set(s1);
-	s2 = cachemgr_ssess_get(s1->session_id, s1->session_id_length);
+	session_id = SSL_SESSION_get_id(s1, &len);
+	s2 = cachemgr_ssess_get(session_id, len);
 	fail_unless(!!s2, "cache returned no session");
 	fail_unless(s2 != s1, "cache returned same pointer");
 	SSL_SESSION_free(s1);
@@ -85,12 +89,15 @@ END_TEST
 START_TEST(cache_ssess_02)
 {
 	SSL_SESSION *s1, *s2;
+	const unsigned char* session_id;
+	unsigned int len;
 
 	s1 = ssl_session_from_file(TMP_SESS_FILE);
 	fail_unless(!!s1, "creating session failed");
 	fail_unless(ssl_session_is_valid(s1), "session invalid");
 
-	s2 = cachemgr_ssess_get(s1->session_id, s1->session_id_length);
+	session_id = SSL_SESSION_get_id(s1, &len);
+	s2 = cachemgr_ssess_get(session_id, len);
 	fail_unless(s2 == NULL, "session was already in empty cache");
 	SSL_SESSION_free(s1);
 }
@@ -99,6 +106,8 @@ END_TEST
 START_TEST(cache_ssess_03)
 {
 	SSL_SESSION *s1, *s2;
+	const unsigned char* session_id;
+	unsigned int len;
 
 	s1 = ssl_session_from_file(TMP_SESS_FILE);
 	fail_unless(!!s1, "creating session failed");
@@ -106,15 +115,19 @@ START_TEST(cache_ssess_03)
 
 	cachemgr_ssess_set(s1);
 	cachemgr_ssess_del(s1);
-	s2 = cachemgr_ssess_get(s1->session_id, s1->session_id_length);
+	session_id = SSL_SESSION_get_id(s1, &len);
+	s2 = cachemgr_ssess_get(session_id, len);
 	fail_unless(s2 == NULL, "cache returned deleted session");
 	SSL_SESSION_free(s1);
 }
 END_TEST
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 START_TEST(cache_ssess_04)
 {
 	SSL_SESSION *s1, *s2;
+	const unsigned char* session_id;
+	unsigned int len;
 
 	s1 = ssl_session_from_file(TMP_SESS_FILE);
 	fail_unless(!!s1, "creating session failed");
@@ -123,7 +136,8 @@ START_TEST(cache_ssess_04)
 	fail_unless(s1->references == 1, "refcount != 1");
 	cachemgr_ssess_set(s1);
 	fail_unless(s1->references == 1, "refcount != 1");
-	s2 = cachemgr_ssess_get(s1->session_id, s1->session_id_length);
+	session_id = SSL_SESSION_get_id(s1, &len);
+	s2 = cachemgr_ssess_get(session_id, len);
 	fail_unless(s1->references == 1, "refcount != 1");
 	fail_unless(!!s2, "cache returned no session");
 	fail_unless(s2->references == 1, "refcount != 1");
@@ -137,6 +151,7 @@ START_TEST(cache_ssess_04)
 	SSL_SESSION_free(s2);
 }
 END_TEST
+#endif
 
 Suite *
 cachessess_suite(void)
@@ -151,7 +166,9 @@ cachessess_suite(void)
 	tcase_add_test(tc, cache_ssess_01);
 	tcase_add_test(tc, cache_ssess_02);
 	tcase_add_test(tc, cache_ssess_03);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	tcase_add_test(tc, cache_ssess_04);
+#endif
 	suite_add_tcase(s, tc);
 
 	return s;
