@@ -40,7 +40,7 @@
 #include "cachemgr.h"
 #include "sys.h"
 #include "log.h"
-#include "version.h"
+#include "build.h"
 #include "defaults.h"
 
 #include <stdlib.h>
@@ -70,8 +70,9 @@ extern int daemon(int, int);
 static void
 main_version(void)
 {
-	fprintf(stderr, "%s %s (built %s)\n", PNAME, version, build_date);
-	if (strlen(version) < 5) {
+	fprintf(stderr, "%s %s (built %s)\n",
+	                PKGLABEL, build_version, build_date);
+	if (strlen(build_version) < 5) {
 		/*
 		 * Note to package maintainers:  If you break the version
 		 * string in your build, it will be impossible to provide
@@ -100,8 +101,8 @@ main_version(void)
 	if (build_info[0]) {
 		fprintf(stderr, "Build info: %s\n", build_info);
 	}
-	if (features[0]) {
-		fprintf(stderr, "Features: %s\n", features);
+	if (build_features[0]) {
+		fprintf(stderr, "Features: %s\n", build_features);
 	}
 	nat_version();
 	fprintf(stderr, "Local process info support: ");
@@ -137,6 +138,8 @@ main_usage(void)
 "  -O          deny all OCSP requests on all proxyspecs\n"
 "  -P          passthrough SSL connections if they cannot be split because of\n"
 "              client cert auth or no matching cert and no CA (default: drop)\n"
+"  -a pemfile  use cert from pemfile when destination requests client certs\n"
+"  -b pemfile  use key from pemfile when destination requests client certs\n"
 #ifndef OPENSSL_NO_DH
 "  -g pemfile  use DH group params from pemfile (default: keyfiles or auto)\n"
 #define OPT_g "g:"
@@ -214,7 +217,7 @@ main_usage(void)
 		warn = "";
 	}
 
-	fprintf(stderr, usagefmt, BNAME, dflt, BNAME, warn);
+	fprintf(stderr, usagefmt, build_pkgname, dflt, build_pkgname, warn);
 }
 
 /*
@@ -306,8 +309,8 @@ main(int argc, char *argv[])
 		natengine = NULL;
 	}
 
-	while ((ch = getopt(argc, argv, OPT_g OPT_G OPT_Z OPT_i "k:c:C:K:t:"
-	                    "OPs:r:R:e:Eu:m:j:p:l:L:S:F:M:dDVhW:w:q:")) != -1) {
+	while ((ch = getopt(argc, argv, OPT_g OPT_G OPT_Z OPT_i "k:c:C:K:t:OPa:"
+	                    "b:s:r:R:e:Eu:m:j:p:l:L:S:F:M:dDVhW:w:q:")) != -1) {
 		switch (ch) {
 			case 'c':
 				if (opts->cacrt)
@@ -428,6 +431,40 @@ main(int argc, char *argv[])
 			case 'P':
 				opts->passthrough = 1;
 				break;
+			case 'a':
+				if (opts->clientcrt)
+					X509_free(opts->clientcrt);
+				opts->clientcrt = ssl_x509_load(optarg);
+				if (!opts->clientcrt) {
+					fprintf(stderr, "%s: error loading cli"
+					                "ent cert from '%s':\n",
+					                argv0, optarg);
+					if (errno) {
+						fprintf(stderr, "%s\n",
+						        strerror(errno));
+					} else {
+						ERR_print_errors_fp(stderr);
+					}
+					exit(EXIT_FAILURE);
+				}
+				break;
+			case 'b':
+				if (opts->clientkey)
+					EVP_PKEY_free(opts->clientkey);
+				opts->clientkey = ssl_key_load(optarg);
+				if (!opts->clientkey) {
+					fprintf(stderr, "%s: error loading cli"
+					                "ent key from '%s':\n",
+					                argv0, optarg);
+					if (errno) {
+						fprintf(stderr, "%s\n",
+						        strerror(errno));
+					} else {
+						ERR_print_errors_fp(stderr);
+					}
+					exit(EXIT_FAILURE);
+				}
+				break;
 #ifndef OPENSSL_NO_DH
 			case 'g':
 				if (opts->dh)
