@@ -1,28 +1,29 @@
-/*
+/*-
  * SSLsplit - transparent SSL/TLS interception
- * Copyright (c) 2009-2016, Daniel Roethlisberger <daniel@roe.ch>
+ * https://www.roe.ch/SSLsplit
+ *
+ * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
- * http://www.roe.ch/SSLsplit
  *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions, and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "nat.h"
@@ -66,6 +67,8 @@
 #include <limits.h>
 #include <linux/netfilter_ipv4.h>
 #include <linux/netfilter_ipv6.h>
+#include <linux/if.h>
+#include <linux/netfilter_ipv6/ip6_tables.h>
 #endif /* HAVE_NETFILTER */
 
 
@@ -124,6 +127,10 @@ nat_pf_lookup_cb(struct sockaddr *dst_addr, socklen_t *dst_addrlen,
 #define sport sxport.port
 #define dport dxport.port
 #define rdport rdxport.port
+#ifdef v4addr /* XNU 4570.1.46 and newer */
+#define v4 v4addr
+#define v6 v6addr
+#endif /* XNU 4570.1.46 and newer */
 #endif /* __APPLE__ */
 	struct sockaddr_storage our_addr;
 	socklen_t our_addrlen;
@@ -196,6 +203,10 @@ nat_pf_lookup_cb(struct sockaddr *dst_addr, socklen_t *dst_addrlen,
 #undef sport
 #undef dport
 #undef rdport
+#ifdef v4addr /* XNU 4570.1.46 and newer */
+#undef v4
+#undef v6
+#endif /* XNU 4570.1.46 and newer */
 #endif /* __APPLE__ */
 }
 #endif /* HAVE_PF */
@@ -313,12 +324,9 @@ nat_ipfilter_lookup_cb(struct sockaddr *dst_addr, socklen_t *dst_addrlen,
 
 #ifdef HAVE_NETFILTER
 /*
- * It seems that SO_ORIGINAL_DST only works for IPv4 and that there
- * is no IPv6 equivalent yet.  Someone please port pf to Linux...
- *
- * http://lists.netfilter.org/pipermail/netfilter/2007-July/069259.html
- *
- * It looks like TPROXY is the only way to go on Linux with IPv6.
+ * Linux commit 121d1e0941e05c64ee4223064dd83eb24e871739 adding
+ * IP6T_SO_ORIGINAL_DST was first released as part of Linux v3.8-rc1 in 2012.
+ * Before that, this interface only supported IPv4.
  */
 static int
 nat_netfilter_lookup_cb(struct sockaddr *dst_addr, socklen_t *dst_addrlen,
@@ -327,16 +335,29 @@ nat_netfilter_lookup_cb(struct sockaddr *dst_addr, socklen_t *dst_addrlen,
 {
 	int rv;
 
-	if (src_addr->sa_family != AF_INET) {
+	if (src_addr->sa_family == AF_INET) {
+		rv = getsockopt(s, SOL_IP, SO_ORIGINAL_DST,
+		                dst_addr, dst_addrlen);
+		if (rv == -1) {
+			log_err_printf("Error from getsockopt("
+			               "SO_ORIGINAL_DST): %s\n",
+			               strerror(errno));
+		}
+	} else {
+#ifdef IP6T_SO_ORIGINAL_DST
+		rv = getsockopt(s, SOL_IPV6, IP6T_SO_ORIGINAL_DST,
+		                dst_addr, dst_addrlen);
+		if (rv == -1) {
+			log_err_printf("Error from getsockopt("
+			               "IP6T_SO_ORIGINAL_DST): %s\n",
+			               strerror(errno));
+		}
+#else /* !IP6T_SO_ORIGINAL_DST */
 		log_err_printf("The netfilter NAT engine only "
-		               "supports IPv4 state lookups\n");
+		               "supports IPv4 state lookups on "
+		               "this version of Linux\n");
 		return -1;
-	}
-
-	rv = getsockopt(s, SOL_IP, SO_ORIGINAL_DST, dst_addr, dst_addrlen);
-	if (rv == -1) {
-		log_err_printf("Error from getsockopt(SO_ORIGINAL_DST): %s\n",
-		               strerror(errno));
+#endif /* !IP6T_SO_ORIGINAL_DST */
 	}
 	return rv;
 }
@@ -431,7 +452,11 @@ struct engine engines[] = {
 #endif /* HAVE_IPFILTER */
 #ifdef HAVE_NETFILTER
 	{
+#ifdef IP6T_SO_ORIGINAL_DST
+		"netfilter", 1, 0,
+#else /* !IP6T_SO_ORIGINAL_DST */
 		"netfilter", 0, 0,
+#endif /* !IP6T_SO_ORIGINAL_DST */
 		NULL, NULL, NULL,
 		nat_netfilter_lookup_cb, NULL
 	},
@@ -640,16 +665,11 @@ nat_version(void)
 #else /* !IP_TRANSPARENT */
 	fprintf(stderr, " !IP_TRANSPARENT");
 #endif /* !IP_TRANSPARENT */
-#ifdef SOL_IPV6
-	fprintf(stderr, " SOL_IPV6");
-#else /* !SOL_IPV6 */
-	fprintf(stderr, " !SOL_IPV6");
-#endif /* !SOL_IPV6 */
-#ifdef IPV6_ORIGINAL_DST
-	fprintf(stderr, " IPV6_ORIGINAL_DST");
-#else /* !IPV6_ORIGINAL_DST */
-	fprintf(stderr, " !IPV6_ORIGINAL_DST");
-#endif /* !IPV6_ORIGINAL_DST */
+#ifdef IP6T_SO_ORIGINAL_DST
+	fprintf(stderr, " IP6T_SO_ORIGINAL_DST");
+#else /* !IP6T_SO_ORIGINAL_DST */
+	fprintf(stderr, " !IP6T_SO_ORIGINAL_DST");
+#endif /* !IP6T_SO_ORIGINAL_DST */
 	fprintf(stderr, "\n");
 #endif /* HAVE_NETFILTER */
 }
