@@ -1,28 +1,29 @@
-/*
+/*-
  * SSLsplit - transparent SSL/TLS interception
- * Copyright (c) 2009-2016, Daniel Roethlisberger <daniel@roe.ch>
+ * https://www.roe.ch/SSLsplit
+ *
+ * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
- * http://www.roe.ch/SSLsplit
  *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions, and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "attrib.h"
@@ -71,11 +72,15 @@ static char *argv11[] = {
 };
 static char *argv12[] = {
 	"autossl", "127.0.0.1", "10025", "127.0.0.2", "25",
-	"https", "127.0.0.1", "10443", "127.0.0.2", "443",
+	"https", "127.0.0.1", "10443", "127.0.0.2", "443"
 };
 static char *argv13[] = {
 	"autossl", "127.0.0.1", "10025", "sni", "25"
 };
+static char *argv14[] = {
+	"https", "127.0.0.1", "10443",
+	"autossl", "127.0.0.1", "10025", "127.0.0.2", "25"
+};
 
 #define NATENGINE "pf"
 
@@ -453,6 +458,40 @@ START_TEST(proxyspec_parse_17)
 }
 END_TEST
 
+START_TEST(proxyspec_parse_18)
+{
+	proxyspec_t *spec;
+	int argc = 8;
+	char **argv = argv14;
+
+	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	fail_unless(!!spec, "failed to parse spec");
+	fail_unless(!spec->ssl, "SSL");
+	fail_unless(!spec->http, "HTTP");
+	fail_unless(spec->upgrade, "not Upgrade");
+	fail_unless(spec->listen_addrlen == sizeof(struct sockaddr_in),
+	            "not IPv4 listen addr");
+	fail_unless(spec->connect_addrlen == sizeof(struct sockaddr_in),
+	            "not IPv4 connect addr");
+	fail_unless(!spec->sni_port, "SNI port is set");
+	fail_unless(!spec->natengine, "natengine is set");
+	fail_unless(!spec->natlookup, "natlookup() is set");
+	fail_unless(!spec->natsocket, "natsocket() is set");
+	fail_unless(!!spec->next, "next is not set");
+	fail_unless(spec->next->ssl, "not SSL");
+	fail_unless(spec->next->http, "not HTTP");
+	fail_unless(!spec->next->upgrade, "Upgrade");
+	fail_unless(spec->next->listen_addrlen == sizeof(struct sockaddr_in),
+	            "not IPv4 listen addr");
+	fail_unless(!spec->next->connect_addrlen, "connect addr set");
+	fail_unless(!spec->next->sni_port, "SNI port is set");
+	fail_unless(!!spec->next->natengine, "natengine is not set");
+	fail_unless(!spec->next->natlookup, "natlookup() is set");
+	fail_unless(!spec->next->natsocket, "natsocket() is set");
+	proxyspec_free(spec);
+}
+END_TEST
+
 START_TEST(opts_debug_01)
 {
 	opts_t *opts;
@@ -497,6 +536,7 @@ opts_suite(void)
 	tcase_add_test(tc, proxyspec_parse_15);
 	tcase_add_test(tc, proxyspec_parse_16);
 	tcase_add_exit_test(tc, proxyspec_parse_17, EXIT_FAILURE);
+	tcase_add_test(tc, proxyspec_parse_18);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("opts_debug");
