@@ -880,6 +880,36 @@ ssl_x509_serial_copyrand(X509 *dstcrt, X509 *srccrt)
 	return 0;
 }
 
+/*
+ * Returns the appropriate key usage strings for the type of server key.
+ * Return value should conceptually be const, but OpenSSL does not use const
+ * appropriately.
+ */
+static char *
+ssl_key_usage_for_key(EVP_PKEY *key)
+{
+	switch (EVP_PKEY_type(EVP_PKEY_base_id(key))) {
+#ifndef OPENSSL_NO_RSA
+	case EVP_PKEY_RSA:
+		return "keyEncipherment,digitalSignature";
+#endif /* !OPENSSL_NO_RSA */
+#ifndef OPENSSL_NO_DH
+	case EVP_PKEY_DH:
+		return "keyAgreement";
+#endif /* !OPENSSL_NO_DH */
+#ifndef OPENSSL_NO_DSA
+	case EVP_PKEY_DSA:
+		return "digitalSignature";
+#endif /* !OPENSSL_NO_DSA */
+#ifndef OPENSSL_NO_ECDSA
+	case EVP_PKEY_EC:
+		return "digitalSignature,keyAgreement";
+#endif /* !OPENSSL_NO_ECDSA */
+	default:
+		return "keyEncipherment,keyAgreement,digitalSignature";
+	}
+}
+
 /*
  * Create a fake X509v3 certificate, signed by the provided CA,
  * based on the original certificate retrieved from the real server.
@@ -933,12 +963,9 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 	if (rv == -1)
 		goto errout;
 
-	rv = ssl_x509_v3ext_copy_by_nid(crt, origcrt,
-	                                NID_key_usage);
-	if (rv == 0)
-		rv = ssl_x509_v3ext_add(&ctx, crt, "keyUsage",
-		                                   "digitalSignature,"
-		                                   "keyEncipherment");
+	/* key usage depends on the key type, do not copy from original */
+	rv = ssl_x509_v3ext_add(&ctx, crt, "keyUsage",
+	                        ssl_key_usage_for_key(key));
 	if (rv == -1)
 		goto errout;
 
@@ -1007,7 +1034,7 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 		}
 	}
 #ifdef DEBUG_CERTIFICATE
-	ssl_x509_v3ext_add(&ctx, crt, "nsComment", "Generated by " PNAME);
+	ssl_x509_v3ext_add(&ctx, crt, "nsComment", "Generated by " PKGLABEL);
 #endif /* DEBUG_CERTIFICATE */
 
 	const EVP_MD *md;
@@ -1021,6 +1048,9 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 		case NID_ripemd160WithRSA:
 			md = EVP_ripemd160();
 			break;
+		case NID_sha1WithRSAEncryption:
+			md = EVP_sha1();
+			break;
 		case NID_sha224WithRSAEncryption:
 			md = EVP_sha224();
 			break;
@@ -1038,9 +1068,8 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 			md = EVP_sha();
 			break;
 #endif /* !OPENSSL_NO_SHA0 */
-		case NID_sha1WithRSAEncryption:
 		default:
-			md = EVP_sha1();
+			md = EVP_sha256();
 			break;
 		}
 		break;
@@ -1048,6 +1077,10 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 #ifndef OPENSSL_NO_DSA
 	case EVP_PKEY_DSA:
 		switch (X509_get_signature_nid(origcrt)) {
+		case NID_dsaWithSHA1:
+		case NID_dsaWithSHA1_2:
+			md = EVP_sha1();
+			break;
 		case NID_dsa_with_SHA224:
 			md = EVP_sha224();
 			break;
@@ -1059,10 +1092,8 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 			md = EVP_sha();
 			break;
 #endif /* !OPENSSL_NO_SHA0 */
-		case NID_dsaWithSHA1:
-		case NID_dsaWithSHA1_2:
 		default:
-			md = EVP_sha1();
+			md = EVP_sha256();
 			break;
 		}
 		break;
@@ -1070,6 +1101,9 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 #ifndef OPENSSL_NO_ECDSA
 	case EVP_PKEY_EC:
 		switch (X509_get_signature_nid(origcrt)) {
+		case NID_ecdsa_with_SHA1:
+			md = EVP_sha1();
+			break;
 		case NID_ecdsa_with_SHA224:
 			md = EVP_sha224();
 			break;
@@ -1082,9 +1116,8 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
 		case NID_ecdsa_with_SHA512:
 			md = EVP_sha512();
 			break;
-		case NID_ecdsa_with_SHA1:
 		default:
-			md = EVP_sha1();
+			md = EVP_sha256();
 			break;
 		}
 		break;
