@@ -1,28 +1,29 @@
-/*
+/*-
  * SSLsplit - transparent SSL/TLS interception
- * Copyright (c) 2009-2016, Daniel Roethlisberger <daniel@roe.ch>
+ * https://www.roe.ch/SSLsplit
+ *
+ * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
- * http://www.roe.ch/SSLsplit
  *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions, and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "sys.h"
@@ -214,15 +215,23 @@ sys_pidf_write(int fd)
 {
 	char pidbuf[4*sizeof(pid_t)];
 	int rv;
+	ssize_t n;
 
 	rv = snprintf(pidbuf, sizeof(pidbuf), "%d\n", getpid());
 	if (rv == -1 || rv >= (int)sizeof(pidbuf))
 		return -1;
 
-	write(fd, pidbuf, strlen(pidbuf));
-	fsync(fd);
+	n = write(fd, pidbuf, strlen(pidbuf));
+	if (n < (ssize_t)strlen(pidbuf))
+		return -1;
 
-	fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
+	rv = fsync(fd);
+	if (rv == -1)
+		return -1;
+
+	rv = fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
+	if (rv == -1)
+		return -1;
 
 	return 0;
 }
@@ -521,7 +530,6 @@ sys_mkpath(const char *path, mode_t mode)
  * Iterate over all files in a directory hierarchy, calling the callback
  * cb for each file, passing the filename and arg as arguments.  Files and
  * directories beginning with a dot are skipped, symlinks are followed.
- * FIXME - there is no facility to return errors from the file handlers
  */
 int
 sys_dir_eachfile(const char *dirname, sys_dir_eachfile_cb_t cb, void *arg)
@@ -529,6 +537,7 @@ sys_dir_eachfile(const char *dirname, sys_dir_eachfile_cb_t cb, void *arg)
 	FTS *tree;
 	FTSENT *node;
 	char * paths[2];
+	int rv = 0;
 
 	paths[1] = NULL;
 	paths[0] = strdup(dirname);
@@ -539,25 +548,32 @@ sys_dir_eachfile(const char *dirname, sys_dir_eachfile_cb_t cb, void *arg)
 	if (!tree) {
 		log_err_printf("Cannot open directory '%s': %s\n",
 		               dirname, strerror(errno));
-		return -1;
+		rv = -1;
+		goto out1;
 	}
 
 	while ((node = fts_read(tree))) {
 		if (node->fts_level > 0 && node->fts_name[0] == '.')
 			fts_set(tree, node, FTS_SKIP);
 		else if (node->fts_info & FTS_F) {
-			cb(node->fts_path, arg);
+			rv = cb(node->fts_path, arg);
+			if (rv == -1)
+				goto out2;
 		}
 	}
 	if (errno) {
 		log_err_printf("Error reading directory entry: %s\n",
 		               strerror(errno));
-		return -1;
+		rv = -1;
+		goto out2;
 	}
+
+out2:
 	fts_close(tree);
 
+out1:
 	free(paths[0]);
-	return 0;
+	return rv;
 }
 
 /*
@@ -699,20 +715,22 @@ sys_afunix_str(struct sockaddr *addr, socklen_t addrlen)
 {
 	struct sockaddr_un *sun = (struct sockaddr_un *)addr;
 	char *name;
+	int rv;
 
 	if (addrlen == sizeof(sa_family_t)) {
-		asprintf(&name, "unnmd");
+		rv = asprintf(&name, "unnmd");
 	} else if (sun->sun_path[0] == '\0') {
 		/* abstract sockets is a Linux feature */
-		asprintf(&name, "abstr:%02x:%02x:%02x:%02x",
-				sun->sun_path[1],
-				sun->sun_path[2],
-				sun->sun_path[3],
-				sun->sun_path[4]);
+		rv = asprintf(&name, "abstr:%02x:%02x:%02x:%02x",
+		                     sun->sun_path[1],
+		                     sun->sun_path[2],
+		                     sun->sun_path[3],
+		                     sun->sun_path[4]);
 	} else {
-		asprintf(&name, "pname:%s", sun->sun_path);
+		rv = asprintf(&name, "pname:%s", sun->sun_path);
 	}
-
+	if (rv == -1)
+		name = NULL;
 	return name;
 }
 
@@ -759,7 +777,7 @@ sys_dump_fds(void)
 		}
 
 		if ((st.st_mode & S_IFMT) == S_IFSOCK) {
-			int lrv, frv;
+			int lrv, frv, arv;
 			struct sockaddr_storage lss, fss;
 			socklen_t lsslen = sizeof(lss);
 			socklen_t fsslen = sizeof(fss);
@@ -779,8 +797,11 @@ sys_dump_fds(void)
 					        &host, &port) != 0) {
 						laddrstr = strdup("?");
 					} else {
-						asprintf(&laddrstr, "[%s]:%s",
-						         host, port);
+						arv = asprintf(&laddrstr,
+						               "[%s]:%s",
+						               host, port);
+						if (arv == -1)
+							laddrstr = NULL;
 						free(host);
 						free(port);
 					}
@@ -795,8 +816,11 @@ sys_dump_fds(void)
 					        &host, &port) != 0) {
 						faddrstr = strdup("?");
 					} else {
-						asprintf(&faddrstr, "[%s]:%s",
-						         host, port);
+						arv = asprintf(&faddrstr,
+						               "[%s]:%s",
+						               host, port);
+						if (arv == -1)
+							faddrstr = NULL;
 						free(host);
 						free(port);
 					}
